package Loove 
{
	import flash.geom.Point;
	import flash.utils.Timer;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.Graphic;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.Sfx;
	
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	
	import net.flashpunk.FP;
	

	public class Player extends Entity
	{
		[Embed(source = '../../assets/Loove_Player_1UP_Sheet_v4.png')] private const PLAYER0:Class;
		[Embed(source = '../../assets/Loove_Player_2UP_Sheet_v4.png')] private const PLAYER1:Class;
		
		[Embed(source = '../../assets/Loove_Player_Bonding_v1.png')] private const PLAYER_BONDING:Class;
		
		//[Embed(source = '../../assets/Loove_1UP_Heartbeat_1.mp3')] private const HEART_BEAT_1UP_PRESS:Class;
		[Embed(source = '../../assets/Loove_1UP_Heartbeat.mp3')] private const HEART_BEAT_1UP_RELEASE:Class;
		//[Embed(source = '../../assets/Loove_2UP_Heartbeat_1.mp3')] private const HEART_BEAT_2UP_PRESS:Class;
		[Embed(source = '../../assets/Loove_2UP_Heartbeat.mp3')] private const HEART_BEAT_2UP_RELEASE:Class;
		

		private var mIsLeftPlayer:Boolean = true;
		private static var mVerticalSpeed:Number = 0;
		private static var mHorizontalSpeed:Number = 0;
		
		private static var mElapsedHeartbeatPlayer1:Number = 0;
		private static var mElapsedHeartbeatPlayer2:Number = 0;
		private static var mElapsedHeartbeatSync:Number = 0;
		
		private static var mHeartbeatScore:Number;
		
		private static var mBondingOverlay:BondingOverlay = new BondingOverlay();
		
		private var heartBall:Spritemap;
		
		//private var mHeartPressUp1:Sfx = new Sfx(HEART_BEAT_1UP_PRESS);
		private var mHeartReleaseUp1:Sfx = new Sfx(HEART_BEAT_1UP_RELEASE);
		//private var mHeartPressUp2:Sfx = new Sfx(HEART_BEAT_2UP_PRESS);
		private var mHeartReleaseUp2:Sfx = new Sfx(HEART_BEAT_2UP_RELEASE);

		private static const MIN_Y:Number = 400;
		private static const MAX_TIME_DISTORTION:Number = 0.1;
		private static const GLOW_FPS:Number = 30;
		
		private static var mGameStarting:Boolean;

		// C'tor
		public function Player(isLeftPlayer:Boolean) 
		{
			mIsLeftPlayer = isLeftPlayer;
			
			type = "player";
			
			if (isLeftPlayer)
			{
				heartBall = new Spritemap(PLAYER0, 78, 78);
			}
			else
			{
				heartBall = new Spritemap(PLAYER1, 78, 78);
			}
			
			//setHitboxTo(heartBall); // change this after PT adds glows!
			//trace("x,y,originX,originY,centerX,centerY", x, y, originX, originY, centerX, centerY);
			heartBall.x = 20;
			heartBall.y = 15;
			setHitbox(40, 30, 0, 0);
			heartBall.centerOrigin();
			
			heartBall.add("standard", [0], 0 , false);
			heartBall.add("glow", [1, 2, 3, 4], GLOW_FPS, false);
			heartBall.add("noGlow", [4, 3, 2, 1, 0], GLOW_FPS, false);
			
			graphic = heartBall;
			setInputs();
			
			trace("new player is left:", isLeftPlayer);
		}
		
		private function setInputs():void
		{
			Input.define("Player1", Key.A);
			Input.define("Player2", Key.L);
		}
		
				private function InitializeMembers():void 
		{
			mHeartbeatScore = 0;
			mVerticalSpeed = 0;
			mHorizontalSpeed = 0;
		
			mElapsedHeartbeatPlayer1 = 0;
			mElapsedHeartbeatPlayer2 = 0;
			mElapsedHeartbeatSync = 0;
			
			heartBall.play("standard");
		}
		
		override public function added():void
		{
			InitializeMembers();
			if (mIsLeftPlayer)
			{
				x = 167 - halfWidth;
			}
			else
			{
				x = 203 - halfWidth;
			}
			y = 630;
			mGameStarting = true;			
		}
		
		override public function update():void
		{
			if (mGameStarting)
			{
				doStartGame();
				doCheckPlayerColl();
				return;
			}
			
			mElapsedHeartbeatPlayer1 += FP.elapsed;
			mElapsedHeartbeatPlayer2 += FP.elapsed;
			mElapsedHeartbeatSync += FP.elapsed;
			
			var delta:Delta = handlePlayerInput();
			
			updatePlayerPositions(delta);
			
			enforceBorderConstraints();
			
			if (collide("obstacle", x, y))
			{
				(FP.world as DuetWorld).setLevelEndScreen(mHeartbeatScore, new Point(x,y));
			}
			doCheckPlayerColl();
		}
		
		private function doCheckPlayerColl():void
		{
			if (collide("player", x , y))
			{
				var world:DuetWorld = FP.world as DuetWorld;
				
				ShowBonding(world.mPlayer1.x + width - mBondingOverlay.halfWidth , y + halfHeight - mBondingOverlay.halfHeight);
				mHorizontalSpeed = 0;

				// reset x positions
				if (mIsLeftPlayer)
					x = (FP.screen.width / 2) - width
				else
					x = (FP.screen.width / 2)
			}
		}
		
		private function doStartGame():void
		{
			updatePlayerPositions(new Delta(0,-0.15));
			
			if (y <= MIN_Y)
				mGameStarting = false;
		}
		
		private function updatePlayerPositions(delta:Delta):void
		{
			mVerticalSpeed += delta.Vertical;
			
			mHorizontalSpeed += delta.Horizontal;
				
			y += mVerticalSpeed * FP.elapsed;
			
			if (mIsLeftPlayer)
			{								
				x -= mHorizontalSpeed * FP.elapsed;
			}
			else 
			{
				x += mHorizontalSpeed * FP.elapsed;
			}			
		}
		
		private function handlePlayerInput():Delta
		{
			var delta:Delta = new Delta(0,0);
						
			if (Input.pressed("Player1") && mIsLeftPlayer)
			{
				heartBall.play("glow");
				//mHeartPressUp1.play();
			}
			
			if (Input.pressed("Player2") && !mIsLeftPlayer)
			{
				heartBall.play("glow");
				//mHeartPressUp2.play();
			}
				 
			var diff:Number = Math.abs( mElapsedHeartbeatPlayer1 - mElapsedHeartbeatPlayer2);
			
			if (Input.released("Player1") && mIsLeftPlayer)
			{
				heartBall.play("NoGlow");
				mHeartReleaseUp1.play();
				
				// if the difference between the players heartbeats is 
				// smaller than MAX_TIME_DISTORTION seconds they are considered "in sync"
				if (diff < MAX_TIME_DISTORTION)
				{
					delta.Horizontal = -20;
					delta.Vertical = -10;
					mElapsedHeartbeatSync = 0;
					mHeartbeatScore++;
				}
				mElapsedHeartbeatPlayer1 = 0;
			} 
			
			if (Input.released("Player2") && !mIsLeftPlayer)
			{
				heartBall.play("NoGlow");
				mHeartReleaseUp2.play();
				
				var timer:Timer

				// if the difference between the players heartbeats is smaller than 0.2 seconds they are considered "in sync"
				if (diff < MAX_TIME_DISTORTION)
				{
					delta.Horizontal = -40;
					delta.Vertical = -20;
					mElapsedHeartbeatSync = 0;
					mHeartbeatScore++;
				}
				mElapsedHeartbeatPlayer2 = 0;
			}
			
			// Gravity
			if (mElapsedHeartbeatSync > 3)
				delta.Vertical = 1;
			
			delta.Horizontal += 0.3;			
			
			return delta;			
		}
		
		private function enforceBorderConstraints():void
		{
			if (mGameStarting)
				return;
			
			if ( x < 0 )
			{
				x = 0;
				mHorizontalSpeed = 0;
			}
			if ( x > FP.screen.width - width )
			{
				x = FP.screen.width - width;
				mHorizontalSpeed = 0;
			}
			
			if ( y < MIN_Y ) y = MIN_Y;
			
			/*
			if (mIsLeftPlayer)
			{
				if ( x > (FP.screen.width / 2) - width)
				{
					x = (FP.screen.width / 2) - width
					mHorizontalSpeed = 0;
				}
			}
			else
			{
				if ( x < (FP.screen.width / 2))
				{
					x = (FP.screen.width / 2)
					mHorizontalSpeed = 0;
				}
			}
			*/
			
			if (mVerticalSpeed > 20)
				mVerticalSpeed = 20;
			
			if (mHorizontalSpeed > 40)
				mHorizontalSpeed = 40;
			
			if ( y > FP.screen.height)
			{
				(FP.world as DuetWorld).setLevelEndScreen(mHeartbeatScore, null);
			}
		}
		
		private function ShowBonding(x:Number, y:Number):void
		{
			mBondingOverlay.x = x;
			mBondingOverlay.y = y;
			FP.world.add(mBondingOverlay);
		}
	}

}